%module FreeImageWrapper
%javaconst(1);
%include "enumtypeunsafe.swg"
%{
#include "FreeImage.h"
typedef BYTE PIXEL;
typedef BYTE RAW;
typedef unsigned INT31;
typedef BOOL BOOL2EXC;

typedef struct {
    BOOL result;
    jint color;
} ColorAndResult;

typedef struct {
    BOOL result;
    jshort index;
} IndexAndResult;

void ThrowFIRuntimeException(JNIEnv *jenv, const char* msg)
{
    jclass excep;

    (*jenv)->ExceptionClear(jenv);
    excep = (*jenv)->FindClass(jenv, "de/goto3d/freeimage/FreeImageRuntimeException");
    if (excep)
    {
        (*jenv)->ThrowNew(jenv, excep, msg);
    }
}

ColorAndResult GetPixelColor(FIBITMAP *dib, unsigned x, unsigned y)
{
	ColorAndResult car;
    RGBQUAD value;
    car.result = FreeImage_GetPixelColor(dib, x,y, &value);
    if ( car.result )
    {
    	car.color = (jint)(value.rgbReserved<<24|value.rgbRed<<16|value.rgbGreen<<8|value.rgbBlue);
    }
    return car;
}
IndexAndResult GetPixelIndex(FIBITMAP *dib, unsigned x, unsigned y)
{
    IndexAndResult iar;
    BYTE value;
    iar.result = FreeImage_GetPixelIndex(dib, x,y, &value);
    if ( iar.result )
    {
        iar.index = (jshort)value;
    }
    return iar;       
}

BOOL2EXC SetPixelColor(FIBITMAP *dib, unsigned x, unsigned y, jint color)
{
    RGBQUAD value;
    value.rgbReserved	= color >> 24;
    value.rgbRed		= color >> 16 & 0xFF;
    value.rgbGreen		= color >> 8 & 0xFF;
    value.rgbBlue		= color & 0xFF;
    return FreeImage_SetPixelColor(dib, x,y, &value);
}
BOOL2EXC SetPixelIndex(FIBITMAP *dib, unsigned x, unsigned y, jshort index)
{
    BYTE value = (BYTE)index;
    return FreeImage_SetPixelIndex(dib, x,y, &value);
}

%}

/* FreeImage_GetBits typemap entries */
%typemap(out) PIXEL* %{
	jresult = (*jenv)->NewDirectByteBuffer(jenv, result, (jlong)FreeImage_GetPitch(arg1)*FreeImage_GetHeight(arg1));
%}
%typemap(jni) PIXEL* "jobject"
%typemap(jtype) PIXEL* "java.nio.ByteBuffer"
%typemap(jstype) PIXEL* "java.nio.ByteBuffer"
%typemap(javaout) PIXEL* {
    return $jnicall;
  }

/* FreeImage_ConvertFromRawBits/ConvertToRawBits typemap entries */
%typemap(in) RAW* %{
	$1 = (*jenv)->GetDirectBufferAddress(jenv, $input); 
%}
%typemap(jni) RAW* "jobject"
%typemap(jtype) RAW* "java.nio.ByteBuffer"
%typemap(jstype) RAW* "java.nio.ByteBuffer"
%typemap(javain) RAW* "$javainput"

// map BOOL to jboolean
%typemap(jni) BOOL "jboolean"
%typemap(jtype) BOOL "boolean"
%typemap(jstype) BOOL "boolean"
%typemap(out) BOOL %{ $result = (jboolean)$1; %}
%typemap(javaout) BOOL {
	return $jnicall;
  }
%typemap(in) BOOL %{ $1 = (BOOL)$input; %}
%typemap(javain) BOOL "$javainput"

// map BOOL2EXC to void
%typemap(jni) BOOL2EXC "void"
%typemap(jtype) BOOL2EXC "void"
%typemap(jstype) BOOL2EXC "void"
%typemap(out) BOOL2EXC %{ %}
%typemap(javaout) BOOL2EXC {
	$jnicall;
  }

// map INT31 to jint
%typemap(jni) INT31 "jint"
%typemap(jtype) INT31 "int"
%typemap(jstype) INT31 "int"
%typemap(out) INT31 %{ $result = (jint)$1; %}
%typemap(javaout) INT31 {
	return $jnicall;
  }
%typemap(in) INT31 %{ $1 = (unsigned)$input; %}
%typemap(javain) INT31 "$javainput"

// map RGBQUAD to jint
%typemap(jni) RGBQUAD "jint"
%typemap(jtype) RGBQUAD "int"
%typemap(jstype) RGBQUAD "int"
%typemap(out) RGBQUAD %{ $result = (jint)$1; %}
%typemap(javaout) RGBQUAD {
	return $jnicall;
  }
%typemap(in) RGBQUAD %{ $1 = (RGBQUAD)$input; %}
%typemap(javain) RGBQUAD "$javainput"

/* map ColorAndResult */
%typemap(out) ColorAndResult %{
    $result = result.color;
%}
%typemap(jni) ColorAndResult "jint"
%typemap(jtype) ColorAndResult "int"
%typemap(jstype) ColorAndResult "int"
%typemap(javaout) ColorAndResult {
    return $jnicall;
  }

/* map IndexAndResult */
%typemap(out) IndexAndResult %{
    $result = result.index;
%}
%typemap(jni) IndexAndResult "jshort"
%typemap(jtype) IndexAndResult "short"
%typemap(jstype) IndexAndResult "short"
%typemap(javaout) IndexAndResult {
    return $jnicall;
  }

/* exception handling */
%exception GetPixelColor %{
  $action
  if (!result.result) {
    char msg[256];
    sprintf(msg, "GetPixelColor(%d,%d) failed", arg2,arg3);
    ThrowFIRuntimeException(jenv, msg);
  }
%}
%exception GetPixelIndex %{
  $action
  if (!result.result) {
    char msg[256];
    sprintf(msg, "GetPixelIndex(%d,%d) failed", arg2,arg3);
    ThrowFIRuntimeException(jenv, msg);
  }
%}
%exception SetPixelColor %{
  $action
  if (!result) {
    char msg[256];
    sprintf(msg, "SetPixelColor(%d,%d) failed", arg2,arg3);
    ThrowFIRuntimeException(jenv, msg);
  }
%}
%exception SetPixelIndex %{
  $action
  if (!result) {
    char msg[256];
    sprintf(msg, "SetPixelIndex(%d,%d) failed", arg2,arg3);
    ThrowFIRuntimeException(jenv, msg);
  }
%}

#if defined(FREEIMAGE_LIB) || !(defined(_WIN32) || defined(__WIN32__))
#define DLL_API
#define DLL_CALLCONV
#else

#define DLL_CALLCONV __stdcall
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the FREEIMAGE_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// DLL_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef FREEIMAGE_EXPORTS
#define DLL_API __declspec(dllexport)
#else
#define DLL_API __declspec(dllimport)
#endif // FREEIMAGE_EXPORTS
#endif // FREEIMAGE_LIB || !WIN32

// For C compatibility --------------------------------------------------------

#define FI_DEFAULT(x)   = x
#define FI_ENUM(x)      enum x
#define FI_STRUCT(x)    struct x

#ifndef _WINDOWS_
#define _WINDOWS_

#ifndef _MSC_VER
// define portable types for 32-bit / 64-bit OS
#include <inttypes.h>
typedef int32_t BOOL;
typedef uint8_t BYTE;
typedef uint16_t WORD;
typedef uint32_t DWORD;
typedef int32_t LONG;
#else
// MS is not C99 ISO compliant
typedef long BOOL;
typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
typedef long LONG;
#endif // _MSC_VER

#endif // _WINDOWS_

// Important enums ----------------------------------------------------------

/** I/O image format identifiers.
*/
FI_ENUM(FREE_IMAGE_FORMAT) {
	FIF_UNKNOWN = -1,
	FIF_BMP		= 0,
	FIF_ICO		= 1,
	FIF_JPEG	= 2,
	FIF_JNG		= 3,
	FIF_KOALA	= 4,
	FIF_LBM		= 5,
	FIF_IFF = FIF_LBM,
	FIF_MNG		= 6,
	FIF_PBM		= 7,
	FIF_PBMRAW	= 8,
	FIF_PCD		= 9,
	FIF_PCX		= 10,
	FIF_PGM		= 11,
	FIF_PGMRAW	= 12,
	FIF_PNG		= 13,
	FIF_PPM		= 14,
	FIF_PPMRAW	= 15,
	FIF_RAS		= 16,
	FIF_TARGA	= 17,
	FIF_TIFF	= 18,
	FIF_WBMP	= 19,
	FIF_PSD		= 20,
	FIF_CUT		= 21,
	FIF_XBM		= 22,
	FIF_XPM		= 23,
	FIF_DDS		= 24,
	FIF_GIF     = 25,
	FIF_HDR		= 26,
	FIF_FAXG3	= 27,
	FIF_SGI		= 28,
	FIF_EXR		= 29,
	FIF_J2K		= 30,
	FIF_JP2		= 31
};

/** Upsampling / downsampling filters. 
Constants used in FreeImage_Rescale.
*/
FI_ENUM(FREE_IMAGE_FILTER) {
	FILTER_BOX		  = 0,	// Box, pulse, Fourier window, 1st order (constant) b-spline
	FILTER_BICUBIC	  = 1,	// Mitchell & Netravali's two-param cubic filter
	FILTER_BILINEAR   = 2,	// Bilinear filter
	FILTER_BSPLINE	  = 3,	// 4th order (cubic) b-spline
	FILTER_CATMULLROM = 4,	// Catmull-Rom spline, Overhauser spline
	FILTER_LANCZOS3	  = 5	// Lanczos3 filter
};

/** Color quantization algorithms.
Constants used in FreeImage_ColorQuantize.
*/
FI_ENUM(FREE_IMAGE_QUANTIZE) {
    FIQ_WUQUANT = 0,		// Xiaolin Wu color quantization algorithm
    FIQ_NNQUANT = 1			// NeuQuant neural-net quantization algorithm by Anthony Dekker
};

/** Dithering algorithms.
Constants used in FreeImage_Dither.
*/
FI_ENUM(FREE_IMAGE_DITHER) {
    FID_FS			= 0,	// Floyd & Steinberg error diffusion
	FID_BAYER4x4	= 1,	// Bayer ordered dispersed dot dithering (order 2 dithering matrix)
	FID_BAYER8x8	= 2,	// Bayer ordered dispersed dot dithering (order 3 dithering matrix)
	FID_CLUSTER6x6	= 3,	// Ordered clustered dot dithering (order 3 - 6x6 matrix)
	FID_CLUSTER8x8	= 4,	// Ordered clustered dot dithering (order 4 - 8x8 matrix)
	FID_CLUSTER16x16= 5,	// Ordered clustered dot dithering (order 8 - 16x16 matrix)
	FID_BAYER16x16	= 6		// Bayer ordered dispersed dot dithering (order 4 dithering matrix)
};

// Init / Error routines ----------------------------------------------------

DLL_API void DLL_CALLCONV FreeImage_Initialise(BOOL load_local_plugins_only FI_DEFAULT(FALSE));
//DLL_API void DLL_CALLCONV FreeImage_DeInitialise(void);

// Version routines ---------------------------------------------------------

DLL_API const char *DLL_CALLCONV FreeImage_GetVersion(void);
DLL_API const char *DLL_CALLCONV FreeImage_GetCopyrightMessage(void);

// Filetype request routines ------------------------------------------------

DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileType(const char *filename, int size FI_DEFAULT(0));
//DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileTypeU(const wchar_t *filename, int size FI_DEFAULT(0));
//DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileTypeFromHandle(FreeImageIO *io, fi_handle handle, int size FI_DEFAULT(0));
//DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileTypeFromMemory(FIMEMORY *stream, int size FI_DEFAULT(0));

// Load / Save routines -----------------------------------------------------

DLL_API FIBITMAP *DLL_CALLCONV FreeImage_Load(FREE_IMAGE_FORMAT fif, const char *filename, int flags FI_DEFAULT(0));
DLL_API BOOL DLL_CALLCONV FreeImage_Save(FREE_IMAGE_FORMAT fif, FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(0));

// DIB info routines --------------------------------------------------------

DLL_API INT31 DLL_CALLCONV FreeImage_GetColorsUsed(FIBITMAP *dib);
DLL_API INT31 DLL_CALLCONV FreeImage_GetBPP(FIBITMAP *dib);
DLL_API INT31 DLL_CALLCONV FreeImage_GetWidth(FIBITMAP *dib);
DLL_API INT31 DLL_CALLCONV FreeImage_GetHeight(FIBITMAP *dib);
DLL_API INT31 DLL_CALLCONV FreeImage_GetPitch(FIBITMAP *dib);

//DLL_API INT31 DLL_CALLCONV FreeImage_GetTransparencyCount(FIBITMAP *dib);
//DLL_API BYTE * DLL_CALLCONV FreeImage_GetTransparencyTable(FIBITMAP *dib);
//DLL_API void DLL_CALLCONV FreeImage_SetTransparent(FIBITMAP *dib, BOOL enabled);
//DLL_API void DLL_CALLCONV FreeImage_SetTransparencyTable(FIBITMAP *dib, BYTE *table, int count);
DLL_API BOOL DLL_CALLCONV FreeImage_IsTransparent(FIBITMAP *dib);
//DLL_API void DLL_CALLCONV FreeImage_SetTransparentIndex(FIBITMAP *dib, int index);
//DLL_API int DLL_CALLCONV FreeImage_GetTransparentIndex(FIBITMAP *dib);

// Allocate / Clone / Unload routines ---------------------------------------

DLL_API void DLL_CALLCONV FreeImage_Unload(FIBITMAP *dib);

// upsampling / downsampling
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_Rescale(FIBITMAP *dib, int dst_width, int dst_height, FREE_IMAGE_FILTER filter);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_MakeThumbnail(FIBITMAP *dib, int max_pixel_size, BOOL convert FI_DEFAULT(TRUE));

// color manipulation routines (point operations)
DLL_API BOOL DLL_CALLCONV FreeImage_AdjustGamma(FIBITMAP *dib, double gamma);
DLL_API BOOL DLL_CALLCONV FreeImage_AdjustBrightness(FIBITMAP *dib, double percentage);
DLL_API BOOL DLL_CALLCONV FreeImage_AdjustContrast(FIBITMAP *dib, double percentage);
DLL_API BOOL DLL_CALLCONV FreeImage_Invert(FIBITMAP *dib);

// rotation and flipping
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_RotateClassic(FIBITMAP *dib, double angle);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_RotateEx(FIBITMAP *dib, double angle, double x_shift, double y_shift, double x_origin, double y_origin, BOOL use_mask);
DLL_API BOOL DLL_CALLCONV FreeImage_FlipHorizontal(FIBITMAP *dib);
DLL_API BOOL DLL_CALLCONV FreeImage_FlipVertical(FIBITMAP *dib);
//DLL_API BOOL DLL_CALLCONV FreeImage_JPEGTransform(const char *src_file, const char *dst_file, FREE_IMAGE_JPEG_OPERATION operation, BOOL perfect FI_DEFAULT(FALSE));

// Smart conversion routines ------------------------------------------------

DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo4Bits(FIBITMAP *dib);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo8Bits(FIBITMAP *dib);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertToGreyscale(FIBITMAP *dib);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo16Bits555(FIBITMAP *dib);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo16Bits565(FIBITMAP *dib);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo24Bits(FIBITMAP *dib);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo32Bits(FIBITMAP *dib);

DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ColorQuantize(FIBITMAP *dib, FREE_IMAGE_QUANTIZE quantize);
//DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ColorQuantizeEx(FIBITMAP *dib, FREE_IMAGE_QUANTIZE quantize FI_DEFAULT(FIQ_WUQUANT), int PaletteSize FI_DEFAULT(256), int ReserveSize FI_DEFAULT(0), RGBQUAD *ReservePalette FI_DEFAULT(NULL));

DLL_API FIBITMAP *DLL_CALLCONV FreeImage_Threshold(FIBITMAP *dib, BYTE T);
DLL_API FIBITMAP *DLL_CALLCONV FreeImage_Dither(FIBITMAP *dib, FREE_IMAGE_DITHER algorithm);

//DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertToRGBF(FIBITMAP *dib);

//DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertToStandardType(FIBITMAP *src, BOOL scale_linear FI_DEFAULT(TRUE));
//DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertToType(FIBITMAP *src, FREE_IMAGE_TYPE dst_type, BOOL scale_linear FI_DEFAULT(TRUE));

DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertFromRawBits(RAW* bits, int width, int height, int pitch, INT31 bpp, INT31 red_mask, INT31 green_mask, INT31 blue_mask, BOOL topdown FI_DEFAULT(FALSE));
DLL_API void DLL_CALLCONV FreeImage_ConvertToRawBits(RAW *bits, FIBITMAP *dib, int pitch, INT31 bpp, INT31 red_mask, INT31 green_mask, INT31 blue_mask, BOOL topdown FI_DEFAULT(FALSE));

// tone mapping operators
//DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ToneMapping(FIBITMAP *dib, FREE_IMAGE_TMO tmo, double first_param FI_DEFAULT(0), double second_param FI_DEFAULT(0));
//DLL_API FIBITMAP* DLL_CALLCONV FreeImage_TmoDrago03(FIBITMAP *src, double gamma FI_DEFAULT(2.2), double exposure FI_DEFAULT(0));
//DLL_API FIBITMAP* DLL_CALLCONV FreeImage_TmoReinhard05(FIBITMAP *src, double intensity FI_DEFAULT(0), double contrast FI_DEFAULT(0));

// Pixel access routines ----------------------------------------------------

DLL_API PIXEL* DLL_CALLCONV FreeImage_GetBits(FIBITMAP *dib);
ColorAndResult GetPixelColor(FIBITMAP *dib, INT31 x, INT31 y);
IndexAndResult GetPixelIndex(FIBITMAP *dib, INT31 x, INT31 y);
BOOL2EXC SetPixelColor(FIBITMAP *dib, INT31 x, INT31 y, jint color);
BOOL2EXC SetPixelIndex(FIBITMAP *dib, INT31 x, INT31 y, jshort index);

// FreeImage helper routines ------------------------------------------------

DLL_API BOOL DLL_CALLCONV FreeImage_IsLittleEndian(void);

%module FreeImageWrapper
%{
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

%}
