import java.io.File;
import java.nio.ByteBuffer;

import de.goto3d.freeimage.FREE_IMAGE_DITHER;
import de.goto3d.freeimage.FREE_IMAGE_QUANTIZE;
import de.goto3d.freeimage.FREE_IMAGE_FILTER;

import de.goto3d.freeimage.FreeImage;

public class Main {

    public static void main(String[] args) throws Exception {

    	System.out.println("FreeImage V"+FreeImage.getVersion() + "(little endian="+FreeImage.isLittleEndian()+")");
        FreeImage fi= new FreeImage(new File("Ghost_in_the_Shell_003.jpg"));
        int width   = fi.getWidth();
        int pitch   = fi.getPitch();
        int height  = fi.getHeight();
        int bpp     = fi.getBitsPerPixel();

        System.out.println ( width + "x" + height + "x" + bpp + ", pitch = " + pitch );

        System.out.println ( "Color at (0,0) = 0x" + Integer.toHexString(fi.getPixelColor(0, 0)).toUpperCase());
        fi.setPixelColor(100,100, 0x00FF0000);
        fi.setPixelColor(100,101, 0x00FF0000);
        fi.setPixelColor(101,100, 0x00FF0000);
        fi.setPixelColor(101,101, 0x00FF0000);

        int start   = pitch*10;
        int end     = start+(width*bpp/8);
        ByteBuffer b= fi.getPixelBuffer();
        for ( int i = start; i < end; i++ ) {
            b.put(i,(byte)0);
        }

        System.out.println ( "is transparent ? " + fi.isTransparent());

        fi.save(new File("temp.jpg"));
        fi.rotateClassic(45).save(new File("temp_rotated.png"));
        fi.rotateEx(10, 100,0, fi.getWidth()*0.5,fi.getHeight()*0.5, true).save(new File("temp_rotateEx.png"));
        fi.convertTo32Bits().save(new File("temp_32bits.png"));
        fi.convertTo4Bits().save(new File("temp_4bits.png"));
        fi.convertTo8Bits().save(new File("temp_8bits.png"));
        fi.convertToGreyscale().save(new File("temp_gray.png"));
        fi.convertTo16Bits555().save(new File("temp_555.tga"));
        fi.convertTo16Bits565().save(new File("temp_565.tga"));
        fi.convertTo24Bits().save(new File("temp_24bits.png"));
        fi.colorQuantize(FREE_IMAGE_QUANTIZE.FIQ_NNQUANT).save(new File("temp_nnquant.gif"));
        fi.colorQuantize(FREE_IMAGE_QUANTIZE.FIQ_WUQUANT).save(new File("temp_wuquant.gif"));
        fi.threshold(128).save(new File("temp_thresh.png"));
        fi.dither(FREE_IMAGE_DITHER.FID_FS).save(new File("temp_dither.png"));
        fi.makeThumbnail(64, true).save(new File("temp_thumb.png"));
        fi.rescale(2000, 2000, FREE_IMAGE_FILTER.FILTER_LANCZOS3).save(new File("temp_rescaled.png"));
        fi.invert();
        fi.save(new File("temp_inverted.png"));

        // check GC and call to fi unload
        b	= null;
        fi	= null;
        System.gc();

        System.out.println ( "Finished" );
        System.exit(0);
    }
}
